@@ -3,6 +3,8 @@
 
    Copyright (C) 2014 Paul Barker
 
+   SPDX-License-Identifier: GPL-2.0-or-later
+
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2, or (at
@@ -95,13 +97,13 @@ static size_t read_data(struct archive *a, void *buffer, size_t len, int *eof)
         /* We don't know the size read so we have to treat this
          * as an error.
          */
-        opkg_msg(ERROR, "Warning when reading data from archive: %s\n",
-                 archive_error_string(a));
+        opkg_msg(ERROR, "Warning when reading data from archive: %s (errno=%d)\n",
+                 archive_error_string(a), archive_errno(a));
         return 0;
 
     case ARCHIVE_RETRY:
-        opkg_msg(ERROR, "Failed to read data from archive: %s\n",
-                 archive_error_string(a));
+        opkg_msg(ERROR, "Failed to read data from archive: %s (errno=%d)\n",
+                 archive_error_string(a), archive_errno(a));
         if (retries++ < 3) {
             opkg_msg(NOTICE, "Retrying...");
             goto retry;
@@ -109,8 +111,8 @@ static size_t read_data(struct archive *a, void *buffer, size_t len, int *eof)
             return 0;
 
     default:
-        opkg_msg(ERROR, "Failed to read data from archive: %s\n",
-                 archive_error_string(a));
+        opkg_msg(ERROR, "Failed to read data from archive: %s (errno=%d)\n",
+                 archive_error_string(a), archive_errno(a));
         return 0;
     }
 }
@@ -267,8 +269,8 @@ static struct archive_entry *read_header(struct archive *ar, int *eof)
         break;
 
     case ARCHIVE_WARN:
-        opkg_msg(NOTICE, "Warning when reading ar archive header: %s\n",
-                 archive_error_string(ar));
+        opkg_msg(NOTICE, "Warning when reading ar archive header: %s (errno=%d)\n",
+                 archive_error_string(ar), archive_errno(ar));
         break;
 
     case ARCHIVE_EOF:
@@ -277,16 +279,16 @@ static struct archive_entry *read_header(struct archive *ar, int *eof)
         return NULL;
 
     case ARCHIVE_RETRY:
-        opkg_msg(ERROR, "Failed to read archive header: %s\n",
-                 archive_error_string(ar));
+        opkg_msg(ERROR, "Failed to read archive header: %s (errno=%d)\n",
+                 archive_error_string(ar), archive_errno(ar));
         if (retries++ < 3)
             goto retry;
         else
             return NULL;
 
     default:
-        opkg_msg(ERROR, "Failed to read archive header: %s\n",
-                 archive_error_string(ar));
+        opkg_msg(ERROR, "Failed to read archive header: %s (errno=%d)\n",
+                 archive_error_string(ar), archive_errno(ar));
         return NULL;
     }
 
@@ -368,22 +370,22 @@ static struct archive *open_disk(int flags)
 
     r = archive_write_disk_set_options(disk, flags);
     if (r == ARCHIVE_WARN)
-        opkg_msg(NOTICE, "Warning when setting disk options: %s\n",
-                 archive_error_string(disk));
+        opkg_msg(NOTICE, "Warning when setting disk options: %s (errno=%d)\n",
+                 archive_error_string(disk), archive_errno(disk));
     else if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Failed to set disk options: %s\n",
-                 archive_error_string(disk));
+        opkg_msg(ERROR, "Failed to set disk options: %s (errno=%d)\n",
+                 archive_error_string(disk), archive_errno(disk));
         goto err_cleanup;
     }
 
     r = archive_write_disk_set_standard_lookup(disk);
     if (r == ARCHIVE_WARN)
         opkg_msg(NOTICE,
-                 "Warning when setting user/group lookup functions: %s\n",
-                 archive_error_string(disk));
+                 "Warning when setting user/group lookup functions: %s (errno=%d)\n",
+                 archive_error_string(disk), archive_errno(disk));
     else if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Failed to set user/group lookup functions: %s\n",
-                 archive_error_string(disk));
+        opkg_msg(ERROR, "Failed to set user/group lookup functions: %s (errno=%d)\n",
+                 archive_error_string(disk), archive_errno(disk));
         goto err_cleanup;
     }
 
@@ -423,13 +425,15 @@ static int extract_entry(struct archive *a, struct archive_entry *entry,
         break;
 
     case ARCHIVE_WARN:
-        opkg_msg(NOTICE, "Warning when extracting archive entry '%s': %s\n",
-                 archive_entry_pathname(entry), archive_error_string(a));
+        opkg_msg(NOTICE, "Warning when extracting archive entry '%s': %s (errno=%d)\n",
+                 archive_entry_pathname(entry), archive_error_string(a),
+                 archive_errno(a));
         break;
 
     case ARCHIVE_RETRY:
-        opkg_msg(ERROR, "Failed to extract archive entry '%s': %s\n",
-                 archive_entry_pathname(entry), archive_error_string(a));
+        opkg_msg(ERROR, "Failed to extract archive entry '%s': %s (errno=%d)\n",
+                 archive_entry_pathname(entry), archive_error_string(a),
+                 archive_errno(a));
         if (retries++ < 3) {
             opkg_msg(NOTICE, "Retrying...\n");
             goto retry;
@@ -437,8 +441,9 @@ static int extract_entry(struct archive *a, struct archive_entry *entry,
             return -1;
 
     default:
-        opkg_msg(ERROR, "Failed to extract archive entry '%s': %s\n",
-                 archive_entry_pathname(entry), archive_error_string(a));
+        opkg_msg(ERROR, "Failed to extract archive entry '%s': %s (errno=%d)\n",
+                 archive_entry_pathname(entry), archive_error_string(a),
+                 archive_errno(a));
         return -1;
     }
 
@@ -506,8 +511,8 @@ static struct archive *open_outer(const char *filename)
     /* Outer package is in 'ar' format, uncompressed. */
     r = archive_read_support_format_ar(outer);
     if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Ar format not supported: %s\n",
-                 archive_error_string(outer));
+        opkg_msg(ERROR, "Ar format not supported: %s (errno=%d)\n",
+                 archive_error_string(outer), archive_errno(outer));
         goto err_cleanup;
     }
     r = archive_read_support_filter_gzip(outer);
@@ -522,16 +527,16 @@ static struct archive *open_outer(const char *filename)
     }
     r = archive_read_support_format_tar(outer);
     if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Tar format not supported: %s\n",
-             archive_error_string(outer));
+        opkg_msg(ERROR, "Tar format not supported: %s (errno=%d)\n",
+                 archive_error_string(outer), archive_errno(outer));
         goto err_cleanup;
     }
 
     r = archive_read_open_filename(outer, filename, EXTRACT_BUFFER_LEN);
     if (r != ARCHIVE_OK)
     {
-        opkg_msg(ERROR, "Failed to open package '%s': %s\n", filename,
-                    archive_error_string(outer));
+        opkg_msg(ERROR, "Failed to open package '%s': %s (errno=%d)\n",
+                 filename, archive_error_string(outer), archive_errno(outer));
         goto err_cleanup;
     }
     return outer;
@@ -609,18 +614,38 @@ static struct archive *open_inner(struct archive *outer)
     }
 #endif
 
+#if HAVE_ZSTD
+    r = archive_read_support_filter_zstd(inner);
+    if (r == ARCHIVE_WARN) {
+        /* libarchive returns ARCHIVE_WARN if the filter is provided by
+         * an external program.
+         */
+        opkg_msg(INFO, "Zstandard support provided by external program.\n");
+    } else if (r != ARCHIVE_OK) {
+        opkg_msg(ERROR, "Zstandard format not supported.\n");
+        goto err_cleanup;
+    }
+#endif
+
     r = archive_read_support_format_tar(inner);
     if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Tar format not supported: %s\n",
-                 archive_error_string(outer));
+        opkg_msg(ERROR, "Tar format not supported: %s (errno=%d)\n",
+                 archive_error_string(outer), archive_errno(outer));
+        goto err_cleanup;
+    }
+
+    r = archive_read_support_format_empty(inner);
+    if (r != ARCHIVE_OK) {
+        opkg_msg(ERROR, "Empty format not supported: %s (errno=%d)\n",
+                 archive_error_string(inner), archive_errno(inner));
         goto err_cleanup;
     }
 
     r = archive_read_open(inner, data, NULL, inner_read, inner_close);
     if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Failed to open inner archive: %s\n",
-                 archive_error_string(inner));
-        goto err_cleanup;
+        opkg_msg(ERROR, "Failed to open inner archive: %s (errno=%d)\n",
+                 archive_error_string(inner), archive_errno(inner));
+        return NULL;
     }
 
     return inner;
@@ -681,7 +706,7 @@ static struct archive *extract_outer(const char *filename, const char *arname)
 
     inner = open_inner(outer);
     if (!inner)
-        goto err_cleanup;
+        return NULL;
 
     return inner;
 
@@ -710,30 +735,30 @@ static struct archive *open_compressed_file(const char *filename)
          */
         opkg_msg(INFO, "Gzip support provided by external program.\n");
     } else if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Gzip format not supported: %s\n",
-                 archive_error_string(ar));
+        opkg_msg(ERROR, "Gzip format not supported: %s (errno=%d)\n",
+                 archive_error_string(ar), archive_errno(ar));
         goto err_cleanup;
     }
 
     r = archive_read_support_format_raw(ar);
     if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Raw format not supported: %s\n",
-                 archive_error_string(ar));
+        opkg_msg(ERROR, "Raw format not supported: %s (errno=%d)\n",
+                 archive_error_string(ar), archive_errno(ar));
         goto err_cleanup;
     }
 
     r = archive_read_support_format_empty(ar);
     if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Empty format not supported: %s\n",
-                 archive_error_string(ar));
+        opkg_msg(ERROR, "Empty format not supported: %s (errno=%d)\n",
+                 archive_error_string(ar), archive_errno(ar));
         goto err_cleanup;
     }
 
     /* Open input file and prepare for reading. */
     r = archive_read_open_filename(ar, filename, EXTRACT_BUFFER_LEN);
     if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Failed to open compressed file '%s': %s\n", filename,
-                 archive_error_string(ar));
+        opkg_msg(ERROR, "Failed to open compressed file '%s': %s (errno=%d)\n",
+                 filename, archive_error_string(ar), archive_errno(ar));
         goto err_cleanup;
     }
 
@@ -766,7 +791,8 @@ int gz_write_archive(const char *filename, const char *gz_filename)
 
     r = archive_read_next_header2(disk, entry);
     if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Failed to read file: '%s' : %s", filename, archive_error_string(disk));
+        opkg_msg(ERROR, "Failed to read file: '%s' : %s (errno=%d)",
+                 filename, archive_error_string(disk), archive_errno(disk));
         goto cleanup;
     }
 
@@ -775,7 +801,8 @@ int gz_write_archive(const char *filename, const char *gz_filename)
 
     r = archive_write_header(a, entry);
     if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Failed to create compressed file: '%s' : %s", gz_filename, archive_error_string(a));
+        opkg_msg(ERROR, "Failed to create compressed file: '%s' : %s (errno=%d)",
+                 gz_filename, archive_error_string(a), archive_errno(a));
         goto cleanup;
     }
 
@@ -822,6 +849,11 @@ struct opkg_ar *ar_open_pkg_control_archive(const char *filename)
     if (!ar->ar) {
         ar->ar = extract_outer(filename, "control.tar.lz4");
     }
+#endif
+#if HAVE_ZSTD
+    if (!ar->ar) {
+        ar->ar = extract_outer(filename, "control.tar.zst");
+    }
 #endif
     if (!ar->ar) {
         free(ar);
@@ -860,6 +892,12 @@ struct opkg_ar *ar_open_pkg_data_archive(const char *filename)
         ar->ar = extract_outer(filename, "data.tar.lz4");
     }
 #endif
+#if HAVE_ZSTD
+    if (!ar->ar) {
+        ar->ar = extract_outer(filename, "data.tar.zst");
+    }
+#endif
+
     if (!ar->ar) {
         free(ar);
         return NULL;
@@ -874,6 +912,14 @@ struct opkg_ar *ar_open_pkg_data_archive(const char *filename)
     ar->extract_flags = ARCHIVE_EXTRACT_OWNER | ARCHIVE_EXTRACT_PERM |
         ARCHIVE_EXTRACT_TIME | ARCHIVE_EXTRACT_UNLINK | ARCHIVE_EXTRACT_NO_OVERWRITE;
 
+#ifdef ENABLE_ACL
+    ar->extract_flags |= ARCHIVE_EXTRACT_ACL;
+#endif
+
+#ifdef ENABLE_XATTR
+    ar->extract_flags |= ARCHIVE_EXTRACT_FFLAGS | ARCHIVE_EXTRACT_XATTR;
+#endif
+
     if (opkg_config->ignore_uid)
         ar->extract_flags &= ~ARCHIVE_EXTRACT_OWNER;
 
