@@ -1231,6 +1231,15 @@ file_list_t *pkg_get_installed_files(pkg_t * pkg)
             if (*file_name == '/') {
                 file_name++;
             }
+            // now we need to remove the trailing '/' in each entry to
+            // be consistent with pkg_write_filelist() that does the
+            // same.
+            int size = strlen(file_name);
+            if (size > 0 && file_name[size-1] == '/')
+            {
+                file_name[size-1] = '\0';
+            }
+
             sprintf_alloc(&installed_file_name, "%s%s", pkg->dest->root_dir,
                           file_name);
         } else {
@@ -1425,6 +1434,16 @@ void pkg_info_preinstall_check(void)
                 iter = niter, niter = file_list_next(installed_files, iter)) {
             file_info_t *installed_file = (file_info_t *)iter->data;
             file_hash_set_file_owner(installed_file->path, pkg);
+            /*
+             * Ensure that directories or symlink to directories used
+             * by multiple packages are tracked in the dir_hash.
+             * This is later used in remove operations to determine if
+             * the directory or symlink can be deleted.
+             */
+            if(file_is_dir(installed_file->path) || file_is_symlink_to_dir(installed_file->path))
+            {
+                dir_hash_add_ref_count(installed_file->path);
+            }
         }
         pkg_free_installed_files(pkg);
     }
